<?php

namespace TekIntegra;

use Doctrine\DBAL\DriverManager;
use Zeedhi\Framework\Session\Session;
use Teknisa\Libs\Util\Utilities;

class ConnectionFactory extends \Zeedhi\Framework\ORM\ConnectionFactory {
    
    protected static function shouldUseNewCrypt($connectionParams){
        if(isset($connectionParams["newCrypt"]) && $connectionParams["newCrypt"]){
            if(!isset($connectionParams["cryptKey"])) {
                throw new \Exception("The crypt key is required");
            }
            return true;
        } 
        return false;
    }
    
    public static function updateConnectionParams(array $connectionParams, $config = null, $eventManager = null, Session $session = null){
	    $request = explode("/",$_SERVER['REQUEST_URI']);
	    $typeRequest = $request[sizeof($request) - 1];
	    
	    $nrOrg = false;
        
        if ($typeRequest == "recebeDados" || $typeRequest == "reenviaDados" || $typeRequest == "preparaAmbiente" || $typeRequest == "validaConexaoCliente") {
            $params = json_decode(file_get_contents('php://input'), true);
	    
            if (is_array($params) === false) {
				throw new \Exception('Dados não foram enviados corretamente. ('. $params .')');
			} else {
                $nrOrg = $params['NRORGINT'];
			}
        } else {
            if($session) {
                $nrOrg = $session->has('nrorg') ? $session->get('nrorg') : $session->get('NRORG');
    
                //Set current DBNAME in session for use in login session
                if(isset($connectionParams['sessionMode'])) {
                    $encryptedDBName = hash('sha3-512', $connectionParams['dbname']);
                    // $encryptedDBName = self::shouldUseNewCrypt($connectionParams) ? 
                    //         hash('sha3-512', $connectionParams['dbname']) :
                    //             Utilities::legacyEncrypt($connectionParams['dbname'],'zeedhi', "\0");
                    $session->set('CURRENT_DBNAME', preg_replace('/[^A-Za-z0-9\-]/', '', $encryptedDBName));
                } else {
                    $encryptedUser = hash('sha3-512', $connectionParams['user']);
                    // $encryptedUser = self::shouldUseNewCrypt($connectionParams) ? 
                    //         hash('sha3-512', $connectionParams['user']) 
                    //         : Utilities::legacyEncrypt($connectionParams['user'],'zeedhi', "\0");
                    $session->set('CURRENT_DBNAME', preg_replace('/[^A-Za-z0-9\-]/', '',$encryptedUser));
                }
            }
        }
        
        if ($nrOrg && isset($connectionParams['useVpd']) && $connectionParams['useVpd'] == 'true') {
            if(isset($connectionParams['vpdWithWallet']) && $connectionParams['vpdWithWallet'] == 'true') {
                unset($connectionParams['host']);
                unset($connectionParams['port']);
                $connectionParams['user'] = '/';
                $connectionParams['password'] = '';
                $connectionParams['dbname'] = 'USR_ORG_' . $nrOrg;
                $connectionParams['sessionMode'] = -2147483648;
            }else {
                $connectionParams['user'] = 'USR_ORG_' . $nrOrg;
                $connectionParams['password'] = $connectionParams['vpdPassword'];
                if(isset($connectionParams['isWallet']) && $connectionParams['isWallet'] == 'true') {
                    $connectionParams['dbname'] = isset($connectionParams['vpdDBNAME'])
                        ? $connectionParams['vpdDBNAME']
                        : $connectionParams['vpdDBName'];
                    $connectionParams['dbname'] = $connectionParams['vpdDBNAME'];
                    $connectionParams['host'] = $connectionParams['vpdHost'];
                    $connectionParams['port'] = $connectionParams['vpdPort'];
                    $connectionParams['service'] = $connectionParams['vpdService'];
                    unset($connectionParams['sessionMode']);
                }
            }
        }

        if(empty($connectionParams["password"])) {
            $connectionParams["password"] = '';
        } else if(isset($connectionParams["isEncrypted"]) && $connectionParams["isEncrypted"] == 'true') {
            // if(self::shouldUseNewCrypt($connectionParams)) {
                $connectionParams["password"] = self::decrypt($connectionParams["password"], $connectionParams["cryptKey"]);
            // } else {
            //     $connectionParams["password"] = self::legacyDecrypt($connectionParams["password"],'zeedhi',"\0");
            // }
            $connectionParams["isEncrypted"] = false;
        }
        return DriverManager::getConnection($connectionParams, $config, $eventManager);
    }
    
    public static function legacyEncrypt($text, $salt = 'libraries', $padWith = '\0') {
        $encryptedString = trim(
            base64_encode(
                mcrypt_encrypt(
                    MCRYPT_RIJNDAEL_256,
                    str_pad($salt, 32, $padWith),
                    $text,
                    MCRYPT_MODE_ECB,
                    mcrypt_create_iv(mcrypt_get_iv_size(MCRYPT_RIJNDAEL_256, MCRYPT_MODE_ECB), MCRYPT_RAND)
                )
            )
        );
        return $encryptedString;
    }

    public static function legacyDecrypt($text, $salt = 'libraries',  $padWith = '\0') {
        $decryptedString = trim(
            mcrypt_decrypt(
                MCRYPT_RIJNDAEL_256,
                str_pad($salt, 32, $padWith),
                base64_decode($text),
                MCRYPT_MODE_ECB,
                mcrypt_create_iv(mcrypt_get_iv_size(MCRYPT_RIJNDAEL_256, MCRYPT_MODE_ECB), MCRYPT_RAND)
            )
        );
        return $decryptedString;
    }
}